@@ -25,40 +25,6 @@
 package beast.app;
 
 
-import java.awt.BorderLayout;
-import java.awt.Component;
-import java.awt.Dimension;
-import java.io.File;
-import java.io.IOException;
-import java.net.URL;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.List;
-import java.util.concurrent.ExecutorService;
-import java.util.concurrent.Executors;
-
-import javax.swing.BorderFactory;
-import javax.swing.Box;
-import javax.swing.ImageIcon;
-import javax.swing.JButton;
-import javax.swing.JCheckBox;
-import javax.swing.JComboBox;
-import javax.swing.JDialog;
-import javax.swing.JFileChooser;
-import javax.swing.JFrame;
-import javax.swing.JLabel;
-import javax.swing.JOptionPane;
-import javax.swing.JTextField;
-import javax.swing.SwingConstants;
-import javax.swing.UIManager;
-import javax.swing.WindowConstants;
-import javax.swing.border.EtchedBorder;
-import javax.swing.filechooser.FileFilter;
-import javax.xml.parsers.ParserConfigurationException;
-
-import org.json.JSONException;
-import org.xml.sax.SAXException;
-
 import beagle.BeagleFlag;
 import beast.app.beastapp.BeastDialog;
 import beast.app.beastapp.BeastMain;
@@ -68,13 +34,25 @@ import beast.app.util.Version;
 import beast.core.Logger;
 import beast.core.Runnable;
 import beast.core.util.Log;
-import beast.util.AddOnManager;
-import beast.util.JSONParser;
-import beast.util.JSONParserException;
-import beast.util.Randomizer;
-import beast.util.XMLParser;
-import beast.util.XMLParserException;
+import beast.util.*;
 import jam.util.IconUtils;
+import org.json.JSONException;
+import org.xml.sax.SAXException;
+
+import javax.swing.*;
+import javax.swing.border.EtchedBorder;
+import javax.swing.filechooser.FileFilter;
+import javax.xml.parsers.ParserConfigurationException;
+import java.awt.*;
+import java.io.BufferedReader;
+import java.io.File;
+import java.io.FileReader;
+import java.io.IOException;
+import java.net.URL;
+import java.util.*;
+import java.util.List;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
 
 /**
  * Main application for performing MCMC runs.
@@ -112,6 +90,9 @@ public class BeastMCMC {
     public void parseArgs(String[] args) throws IOException, XMLParserException, JSONException {
         int i = 0;
         boolean resume = false;
+        boolean useStrictVersions = false;
+        boolean sampleFromPrior = false;
+        Map<String, String> parserDefinitions = new HashMap<>();
 
         File beastFile = null;
 
@@ -152,6 +133,35 @@ public class BeastMCMC {
                     } else if (args[i].equals("-prefix")) {
                         System.setProperty("file.name.prefix", args[i + 1].trim());
                         i += 2;
+                    } else if (args[i].equals("-D")) {
+                        String [] strs = args[i + 1].split("=",-1);
+                        for (int eqIdx = 0; eqIdx<strs.length-1; eqIdx++) {
+                            int lastCommaIdx = strs[eqIdx].lastIndexOf(",");
+
+                            if (lastCommaIdx != -1 && eqIdx == 0)
+                                throw new IllegalArgumentException("Argument to -D is not well-formed: expecting comma-separated name=value pairs");
+
+                            String name = strs[eqIdx].substring(lastCommaIdx+1);
+
+                            lastCommaIdx = strs[eqIdx+1].lastIndexOf(",");
+                            String value;
+                            if (eqIdx+1 == strs.length-1) {
+                                value = strs[eqIdx+1];
+                            } else {
+                                if (lastCommaIdx == -1)
+                                    throw new IllegalArgumentException("Argument to -D is not well-formed: expecting comma-separated name=value pairs");
+
+                                value = strs[eqIdx+1].substring(0, lastCommaIdx);
+                            }
+                            parserDefinitions.put(name, value);
+            			}
+                        i += 2;
+                    } else if (args[i].equals("-strictversions")) {
+                    	useStrictVersions = true;
+                        i += 1;
+                    } else if (args[i].equals("-sampleFromPrior")) {
+                    	sampleFromPrior = true;
+                        i += 1;
                     }
                     if (i == old) {
                         if (i == args.length - 1) {
@@ -250,14 +260,44 @@ public class BeastMCMC {
             Log.info.println("Resuming from file");
         }
 
-        AddOnManager.loadExternalJars();
+        if (useStrictVersions) {
+        	// grab "required" attribute from beast spec
+            if (beastFile.getPath().toLowerCase().endsWith(".json")) {
+            	throw new IllegalArgumentException("The -strictversions flag is not implemented for JSON files yet (only XML files are supported).");
+            } else {
+                BufferedReader fin = new BufferedReader(new FileReader(beastFile));
+                StringBuffer buf = new StringBuffer();
+                String str = null;
+                int lineCount = 0;
+                while (fin.ready() && lineCount < 100) {
+                    str = fin.readLine();
+                    buf.append(str);
+                    buf.append(' ');
+                }
+                fin.close();
+                str = buf.toString();
+                int start = str.indexOf("required=");
+                if (start < 0) {
+                	throw new IllegalArgumentException("Could not find a 'required' attribute in the XML. Add the required attribute, or run without the -strictversions flag");
+                }
+                char c = str.charAt(start + 9);
+                start += 10;
+                int end = str.indexOf(c, start);
+                String packages = str.substring(start, end);
+                PackageManager.loadExternalJars(packages);
+            }
+        } else {
+            PackageManager.loadExternalJars();
+        }
+        
+
         // parse xml
         Randomizer.setSeed(m_nSeed);
         if (beastFile.getPath().toLowerCase().endsWith(".json")) {
-            m_runnable = new JSONParser().parseFile(beastFile);
+            m_runnable = new JSONParser(parserDefinitions).parseFile(beastFile, sampleFromPrior);
         } else {        	
         	try {
-				m_runnable = new XMLParser().parseFile(beastFile);
+				m_runnable = new XMLParser(parserDefinitions).parseFile(beastFile, sampleFromPrior);
 			} catch (SAXException | ParserConfigurationException e) {
 				throw new IllegalArgumentException(e);
 			}
