@@ -1,11 +1,30 @@
 package beast.app.beauti;
 
 
-import java.awt.BorderLayout;
-import java.awt.Component;
-import java.awt.Container;
-import java.awt.Cursor;
-import java.awt.Toolkit;
+import beast.app.BEASTVersion2;
+import beast.app.beauti.BeautiDoc.ActionOnExit;
+import beast.app.beauti.BeautiDoc.DOC_STATUS;
+import beast.app.draw.BEASTObjectPanel;
+import beast.app.draw.HelpBrowser;
+import beast.app.draw.ModelBuilder;
+import beast.app.draw.MyAction;
+import beast.app.tools.AppLauncher;
+import beast.app.util.Utils;
+import beast.app.util.Utils6;
+import beast.core.BEASTInterface;
+import beast.core.util.Log;
+import beast.evolution.alignment.Alignment;
+import beast.math.distributions.MRCAPrior;
+import beast.util.PackageManager;
+import jam.framework.DocumentFrame;
+import org.w3c.dom.Document;
+import org.w3c.dom.Element;
+
+import javax.swing.*;
+import javax.swing.event.ChangeEvent;
+import javax.swing.event.ChangeListener;
+import javax.xml.parsers.DocumentBuilderFactory;
+import java.awt.*;
 import java.awt.datatransfer.Clipboard;
 import java.awt.datatransfer.ClipboardOwner;
 import java.awt.datatransfer.StringSelection;
@@ -21,52 +40,8 @@ import java.io.PrintStream;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.net.URL;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.HashSet;
+import java.util.*;
 import java.util.List;
-import java.util.Set;
-
-import javax.swing.AbstractAction;
-import javax.swing.Action;
-import javax.swing.Icon;
-import javax.swing.ImageIcon;
-import javax.swing.JCheckBoxMenuItem;
-import javax.swing.JComponent;
-import javax.swing.JDialog;
-import javax.swing.JFrame;
-import javax.swing.JMenu;
-import javax.swing.JMenuBar;
-import javax.swing.JMenuItem;
-import javax.swing.JOptionPane;
-import javax.swing.JPopupMenu;
-import javax.swing.JScrollPane;
-import javax.swing.JTabbedPane;
-import javax.swing.JTextArea;
-import javax.swing.UIManager;
-import javax.swing.WindowConstants;
-import javax.swing.event.ChangeEvent;
-import javax.swing.event.ChangeListener;
-import javax.xml.parsers.DocumentBuilderFactory;
-
-import org.w3c.dom.Document;
-import org.w3c.dom.Element;
-
-import beast.app.BEASTVersion2;
-import beast.app.beauti.BeautiDoc.ActionOnExit;
-import beast.app.beauti.BeautiDoc.DOC_STATUS;
-import beast.app.draw.BEASTObjectPanel;
-import beast.app.draw.HelpBrowser;
-import beast.app.draw.ModelBuilder;
-import beast.app.draw.MyAction;
-import beast.app.tools.AppStore;
-import beast.app.util.Utils;
-import beast.core.BEASTInterface;
-import beast.core.util.Log;
-import beast.evolution.alignment.Alignment;
-import beast.math.distributions.MRCAPrior;
-import beast.util.AddOnManager;
-import jam.framework.DocumentFrame;
 
 
 public class Beauti extends JTabbedPane implements BeautiDocListener {
@@ -117,7 +92,6 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
 
     JCheckBoxMenuItem autoSetClockRate;
     JCheckBoxMenuItem allowLinking;
-    JCheckBoxMenuItem autoUpdateOperatorWeights;
     JCheckBoxMenuItem autoUpdateFixMeanSubstRate;
 
     /**
@@ -171,7 +145,8 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
     Action a_new = new ActionNew();
     public Action a_load = new ActionLoad();
     Action a_template = new ActionTemplate();
-    Action a_addOn = new ActionAddOn();
+    Action a_managePackages = new ActionManagePacakges();
+    Action a_clearClassPath = new ActionClearClassPath();
     Action a_appLauncher = new ActionAppLauncher();
 //    public Action a_import = new ActionImport();
     public Action a_save = new ActionSave();
@@ -403,10 +378,10 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
         } // actionPerformed
     } // ActionTemplate
 
-    class ActionAddOn extends MyAction {
+    class ActionManagePacakges extends MyAction {
         private static final long serialVersionUID = 1;
 
-        public ActionAddOn() {
+        public ActionManagePacakges() {
             super("Manage Packages", "Manage Packages", "package", -1);
         } // c'tor
 
@@ -426,7 +401,24 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
             setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
         } // actionPerformed
     }
+    
+    class ActionClearClassPath extends MyAction {
+        private static final long serialVersionUID = 1;
 
+        public ActionClearClassPath() {
+            super("Clear Class Path", "Clear class path, so it will be refreshed next time BEAUti starts. Only useful when installing packages by hand.", "ccp", -1);
+        } // c'tor
+
+        @Override
+		public void actionPerformed(ActionEvent ae) {
+        	Utils6.saveBeautiProperty("package.path", null);
+        	JOptionPane.showMessageDialog(null, "The class path was cleared.\n"
+        			+ "Next time you start BEAUti, the class path will be re-established.\n"
+        			+ "This is only useful when you install packages by han.d\n"
+        			+ "Otherwise, this is harmless, but onlys potentially slows restarting BEAUti.");
+        } // actionPerformed
+    }
+    
     class ActionAppLauncher extends MyAction {
         private static final long serialVersionUID = 1;
 
@@ -436,7 +428,7 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
 
         @Override
 		public void actionPerformed(ActionEvent ae) {
-        	AppStore.main(new String[]{});
+        	AppLauncher.main(new String[]{});
         } // actionPerformed
     }
 
@@ -610,7 +602,8 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
 		public void actionPerformed(ActionEvent ae) {
             BEASTVersion2 version = new BEASTVersion2();
             JOptionPane.showMessageDialog(null, version.getCredits(),
-                    "About Beauti " + version.getVersionString(), JOptionPane.PLAIN_MESSAGE,
+                    "About Beauti " + version.getVersionString() + 
+                    " Java version " + System.getProperty("java.version"), JOptionPane.PLAIN_MESSAGE,
                     Utils.getIcon(BEAUTI_ICON));
         }
     } // class ActionAbout
@@ -757,14 +750,6 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
             });
         modeMenu.add(allowLinking);
 
-        autoUpdateOperatorWeights = new JCheckBoxMenuItem(
-                "Automatic change operator weights for *BEAST analyses", this.doc.autoUpdateOperatorWeights);
-        autoUpdateOperatorWeights.addActionListener(ae -> {
-                doc.autoUpdateOperatorWeights = autoUpdateOperatorWeights.getState();
-                refreshPanel();
-            });
-        modeMenu.add(autoUpdateOperatorWeights);
-
         autoUpdateFixMeanSubstRate = new JCheckBoxMenuItem(
                 "Automatic set fix mean substitution rate flag", this.doc.autoUpdateFixMeanSubstRate);
         autoUpdateFixMeanSubstRate.addActionListener(ae -> {
@@ -795,7 +780,7 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
         helpMenu.add(a_msgs);
         helpMenu.add(a_citation);
         helpMenu.add(a_viewModel);
-        if (!Utils.isMac()) {
+        if (!Utils.isMac() || Utils6.isMajorLower(Utils6.JAVA_1_8)) {
             helpMenu.add(a_about);
         }
 
@@ -827,7 +812,8 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
         }
         templateMenu.addSeparator();
         templateMenu.add(a_template);
-        fileMenu.add(a_addOn);
+        fileMenu.add(a_managePackages);
+        fileMenu.add(a_clearClassPath);
         fileMenu.add(a_appLauncher);
         fileMenu.addSeparator();
         fileMenu.add(a_save);
@@ -1008,7 +994,7 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
 
 	private List<AbstractAction> getTemplateActions() {
         List<AbstractAction> actions = new ArrayList<>();
-        List<String> beastDirectories = AddOnManager.getBeastDirectories();
+        List<String> beastDirectories = PackageManager.getBeastDirectories();
         for (String dirName : beastDirectories) {
             File dir = new File(dirName + "/templates");
             getTemplateActionForDir(dir, actions);
@@ -1049,7 +1035,7 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
 
     private List<AbstractAction> getWorkDirActions() {
         List<AbstractAction> actions = new ArrayList<>();
-        List<String> beastDirectories = AddOnManager.getBeastDirectories();
+        List<String> beastDirectories = PackageManager.getBeastDirectories();
         Set<String> doneDirs = new HashSet<>();
         for (String dir : beastDirectories) {
         	if (!doneDirs.contains(dir)) {
@@ -1170,6 +1156,8 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
    
 
     public static Beauti main2(String[] args) {
+    	Utils6.startSplashScreen();
+    	Utils6.logToSplashScreen("Initialising BEAUti");
         try {
         	ByteArrayOutputStream baos = null;
             for (String arg : args) {
@@ -1226,7 +1214,7 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
             	}
             }
 
-            AddOnManager.loadExternalJars();
+            PackageManager.loadExternalJars();
             //if (!Utils.isMac()) {
             	Utils.loadUIManager();
             //}
@@ -1239,8 +1227,8 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
             }
 
             final Beauti beauti = new Beauti(doc);
-            
-            if (Utils.isMac()) {
+
+            if (Utils.isMac() && Utils6.isMajorAtLeast(Utils6.JAVA_1_8)) {
                 // set up application about-menu for Mac
                 // Mac-only stuff
                 try {
@@ -1278,16 +1266,23 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
                             return null;
                         }
                     };
-                    jam.mac.Utils.macOSXRegistration(application);
+
+                    // https://github.com/CompEvol/beast2/issues/805
+                    if (Utils6.isMajorAtLeast(Utils6.JAVA_9)) // >= Java 9
+                        beast.app.util.Utils.macOSXRegistration(application);
+                    else // <= Java 8
+                        jam.mac.Utils.macOSXRegistration(application);
                 } catch (Exception e) {
                     // ignore
                 }
-                try {
-                    Class<?> class_ = Class.forName("jam.maconly.OSXAdapter");
-                    Method method = class_.getMethod("enablePrefs", boolean.class);
-                    method.invoke(null, false);
-                } catch (java.lang.NoSuchMethodException e) {
-                    // ignore
+                if (Utils6.isMajorLower(Utils6.JAVA_9)) {
+                    try {
+                        Class<?> class_ = Class.forName("jam.maconly.OSXAdapter");
+                        Method method = class_.getMethod("enablePrefs", boolean.class);
+                        method.invoke(null, false);
+                    } catch (java.lang.NoSuchMethodException e) {
+                        // ignore
+                    }
                 }
             }
             beauti.setUpPanels();
@@ -1402,6 +1397,7 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
             // AWTEvent.ACTION_EVENT_MASK|AWTEvent.ITEM_EVENT_MASK|AWTEvent.MOUSE_EVENT_MASK);
             // // beauti.addPropertyChangeListener(plistener);
 
+        	Utils6.endSplashScreen();
             return beauti;
         } catch (Exception e) {
             e.printStackTrace();
@@ -1411,6 +1407,18 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
 
     public static void main(String[] args) {
         main2(args);
+
+        // check for new packages in the background
+        new Thread() {
+        	public void run() {
+        		String statuString = Utils.getBeautiProperty("package.update.status");
+        		if (statuString == null) {
+        			statuString = PackageManager.UpdateStatus.AUTO_CHECK_AND_ASK.toString(); 
+        		}
+        		PackageManager.UpdateStatus updateStatus = PackageManager.UpdateStatus.valueOf(statuString);
+        		PackageManager.updatePackages(updateStatus, true);
+        	};
+        }.start();
     }
 
 } // class Beauti
