@@ -4,6 +4,8 @@ import java.awt.*;
 import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileReader;
+import java.util.HashMap;
+import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import java.util.regex.PatternSyntaxException;
@@ -44,8 +46,8 @@ public class GuessPatternDialog extends JDialog {
                     "taxonB 0.1\n" +
                     "taxonC 0.2\n" +
                     "\n" +
-                    "where the gap between each taxon name and the numeric value " +
-                    "representing the age must be a TAB.";
+                    "where the gap between each taxon name and its corresponding " +
+                    "trait value must be a TAB.";
 
 
 
@@ -53,10 +55,35 @@ public class GuessPatternDialog extends JDialog {
         canceled, pattern, trait
     };
 
-    public String trait = null;
+    public Map<String,String> traitMap;
 
+    public Map<String,String> getTraitMap() {
+        return traitMap;
+    }
+
+    /**
+     * Constructs and returns a string containing a comma-delimited list of
+     * taxonName=traitValue pairs, as used by the TraitSet initializer.
+     *
+     * This method is deprecated: use getTraitMap instead.
+     *
+     * @return constructed string
+     */
+    @Deprecated
     public String getTrait() {
-        return trait;
+       StringBuilder sb = new StringBuilder();
+
+       boolean isFirst = true;
+       for (String taxonName : traitMap.keySet()) {
+           if (isFirst)
+               isFirst = false;
+           else
+               sb.append(",");
+
+           sb.append(taxonName).append("=").append(traitMap.get(taxonName));
+       }
+
+       return sb.toString();
     }
 
     Component m_parent;
@@ -208,20 +235,8 @@ public class GuessPatternDialog extends JDialog {
 
         JButton btnHelp = new JButton("?");
         btnHelp.setToolTipText("Show format of trait file");
-        btnHelp.addActionListener(e -> {
-	        JOptionPane pane = new JOptionPane() {
-        	    @Override
-        	    public int getMaxCharactersPerLineCount() {
-        	        return 70;
-        	    }
-	        };
-	        pane.setMessage(TRAIT_FILE_HELP_MESSAGE);
-	        pane.setMessageType(JOptionPane.INFORMATION_MESSAGE);
-
-	        JDialog dialog = pane.createDialog(this, "Message");
-	        dialog.setModal(true);
-	        dialog.setVisible(true);
-        });
+        btnHelp.addActionListener(e -> WrappedOptionPane.showWrappedMessageDialog(
+                guessPanel, TRAIT_FILE_HELP_MESSAGE));
         GridBagConstraints gbc_btnHelp = new GridBagConstraints();
         gbc_btnHelp.insets = new Insets(0, 0, 5, 5);
         gbc_btnHelp.gridx = 4;
@@ -231,9 +246,7 @@ public class GuessPatternDialog extends JDialog {
 
         chckbxAddFixedValue = new JCheckBox("Add fixed value");
         chckbxAddFixedValue.setName("Add fixed value");
-        chckbxAddFixedValue.addActionListener(e -> {
-                updateFields();
-            });
+        chckbxAddFixedValue.addActionListener(e -> updateFields());
 
         separator_5 = new JSeparator();
         separator_5.setPreferredSize(new Dimension(5,1));
@@ -552,28 +565,19 @@ public class GuessPatternDialog extends JDialog {
             pattern = textRegExp.getText();
         }
         if (readFromFile.getModel() == group.getSelection()) {
+            traitMap = new HashMap<>();
             try {
                 BufferedReader fin = new BufferedReader(new FileReader(txtFile.getText()));
-                StringBuffer buf = new StringBuffer();
-                // do not eat up header -- it might contain a useful entry, 
-                // but if not, it will not hurt
-                // fin.readLine();
-                // process data
                 while (fin.ready()) {
-                    String str = fin.readLine();
-                    str = str.replaceFirst("\t", "=") + ",";
+                    String[] strArray = fin.readLine().trim().split("\t");
                     // only add entries that are non-empty
-                    if (str.indexOf("=") > 0 && !str.matches("^\\s+=.*$")) {
-                        buf.append(str);
+                    if (strArray.length == 2) {
+                        traitMap.put(strArray[0], strArray[1]);
                     }
                 }
                 fin.close();
-                trait = buf.toString().trim();
-                while (trait.endsWith(",")) {
-                    trait = trait.substring(0, trait.length() - 1).trim();
-                }
-                
-               if (trait.trim().length() == 0) {
+
+               if (traitMap.isEmpty()) {
             	   JOptionPane.showMessageDialog(m_parent, "Could not find trait information in the file. " +
             			   "Perhaps this is not a tab-delimited but space file?");
                }
