@@ -36,7 +36,7 @@ import beast.evolution.tree.TreeInterface;
 import beast.math.distributions.MRCAPrior;
 import beast.math.distributions.OneOnX;
 import beast.math.distributions.Prior;
-import beast.util.AddOnManager;
+import beast.util.PackageManager;
 
 
 
@@ -126,9 +126,9 @@ public class PriorListInputEditor extends ListInputEditor {
         }
         
         if (m_buttonStatus == ButtonStatus.ALL || m_buttonStatus == ButtonStatus.ADD_ONLY) {
-	        addButton = new SmallButton("+", true);
+	        addButton = new SmallButton("+ Add Prior", true);
 	        addButton.setName("addItem");
-	        addButton.setToolTipText("Add item to the list");
+	        addButton.setToolTipText("Add new prior (like an MRCA-prior) to the list of priors");
 	        addButton.addActionListener(e -> {
 	                addItem();
 	            });
@@ -225,7 +225,7 @@ public class PriorListInputEditor extends ListInputEditor {
     	priorProviders.add(new MRCAPriorProvider());
     	
         // build up list of data types
-        List<String> importerClasses = AddOnManager.find(PriorProvider.class, new String[]{"beast.app"});
+        List<String> importerClasses = PackageManager.find(PriorProvider.class, new String[]{"beast.app"});
         for (String _class: importerClasses) {
         	try {
         		if (!_class.startsWith(this.getClass().getName())) {
@@ -249,8 +249,12 @@ public class PriorListInputEditor extends ListInputEditor {
     	if (priorProviders.size() > 1) {
 			// let user choose a PriorProvider
 			List<String> descriptions = new ArrayList<>();
+			List<PriorProvider> availableProviders = new ArrayList<>();
 			for (PriorProvider i : priorProviders) {
-				descriptions.add(i.getDescription());
+				if (i.canProvidePrior(doc)) {
+					descriptions.add(i.getDescription());
+					availableProviders.add(i);
+				}
 			}
 			String option = (String)JOptionPane.showInputDialog(null, "Which prior do you want to add", "Option",
                     JOptionPane.WARNING_MESSAGE, null, descriptions.toArray(), descriptions.get(0));
@@ -258,7 +262,7 @@ public class PriorListInputEditor extends ListInputEditor {
 				return null;
 			}
 			int i = descriptions.indexOf(option);
-			priorProvider = priorProviders.get(i);
+			priorProvider = availableProviders.get(i);
 
     	}
     	
