@@ -24,6 +24,8 @@
 package beast.core;
 
 
+import beast.core.util.Log;
+
 import java.lang.annotation.Annotation;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Field;
@@ -93,6 +95,12 @@ public interface BEASTInterface {
               throw new RuntimeException("Expected a String in " + i + "th argument ");
           }
       }
+      try {
+          validateInputs();
+      } catch (IllegalArgumentException ex) {
+          ex.printStackTrace();
+          throw new RuntimeException("validateInputs() failed! " + ex.getMessage());
+      }
       try {
           initAndValidate();
       } catch (Exception e) {
@@ -397,8 +405,14 @@ public interface BEASTInterface {
      * @throws IllegalArgumentException when validation fails
      */
     default public void validateInputs() {
-        for (final Input<?> input : listInputs()) {
-            input.validate();
+        try {
+            for (final Input<?> input : listInputs()) {
+                input.validate();
+            }
+        } catch (IllegalArgumentException ex) {
+            Log.err.println("Validation error when initializing object " +
+                    this.getClass().getName() + " (id " + getID() + "):");
+            throw ex;
         }
     }
 
@@ -420,6 +434,18 @@ public interface BEASTInterface {
         }
     }
 
+    
+    /** Determine class of all of the inputs of this object 
+     * if that has not already happened
+     */
+    default public void determindClassOfInputs() {
+    	for (Input<?> input : listInputs()) {
+    		if (input.getType() == null) {
+    			input.determineClass(this);
+    		}
+    	}
+    }
+    
 }
 
 
