@@ -50,7 +50,7 @@ import beast.util.Randomizer;
         DOI="10.1371/journal.pcbi.1003537")
 public class MCMC extends Runnable {
 
-    final public Input<Integer> chainLengthInput =
+    final public Input<Long> chainLengthInput =
             new Input<>("chainLength", "Length of the MCMC chain i.e. number of samples taken in main loop",
                     Input.Validate.REQUIRED);
 
@@ -246,7 +246,7 @@ public class MCMC extends Runnable {
         }
     } // init
 
-    public void log(final int sampleNr) {
+    public void log(final long sampleNr) {
         for (final Logger log : loggers) {
             log.log(sampleNr);
         }
@@ -263,7 +263,7 @@ public class MCMC extends Runnable {
     protected double oldLogLikelihood;
     protected double newLogLikelihood;
     protected int burnIn;
-    protected int chainLength;
+    protected long chainLength;
     protected Distribution posterior;
 
     protected List<Logger> loggers;
@@ -306,7 +306,6 @@ public class MCMC extends Runnable {
         logAlpha = 0;
         debugFlag = Boolean.valueOf(System.getProperty("beast.debug"));
 
-
 //        System.err.println("Start state:");
 //        System.err.println(state.toString());
 
@@ -380,91 +379,8 @@ public class MCMC extends Runnable {
         if (burnIn > 0) {
         	Log.warning.println("Please wait while BEAST takes " + burnIn + " pre-burnin samples");
         }
-        for (int sampleNr = -burnIn; sampleNr <= chainLength; sampleNr++) {
-            final int currentState = sampleNr;
-
-            state.store(currentState);
-//            if (m_nStoreEvery > 0 && sample % m_nStoreEvery == 0 && sample > 0) {
-//                state.storeToFile(sample);
-//            	operatorSchedule.storeToFile();
-//            }
-
-            final Operator operator = operatorSchedule.selectOperator();
-
-            if (printDebugInfo) System.err.print("\n" + sampleNr + " " + operator.getName()+ ":");
-
-            final Distribution evaluatorDistribution = operator.getEvaluatorDistribution();
-            Evaluator evaluator = null;
-
-            if (evaluatorDistribution != null) {
-                evaluator = new Evaluator() {
-                    @Override
-                    public double evaluate() {
-                        double logP = 0.0;
-
-                        state.storeCalculationNodes();
-                        state.checkCalculationNodesDirtiness();
-
-                        try {
-                            logP = evaluatorDistribution.calculateLogP();
-                        } catch (Exception e) {
-                            e.printStackTrace();
-                            System.exit(1);
-                        }
-
-                        state.restore();
-                        state.store(currentState);
-
-                        return logP;
-                    }
-                };
-            }
-            final double logHastingsRatio = operator.proposal(evaluator);
-
-            if (logHastingsRatio != Double.NEGATIVE_INFINITY) {
-
-            	if (operator.requiresStateInitialisation()) {
-            		state.storeCalculationNodes();
-            		state.checkCalculationNodesDirtiness();
-            	}
-
-                newLogLikelihood = posterior.calculateLogP();
-
-                logAlpha = newLogLikelihood - oldLogLikelihood + logHastingsRatio; //CHECK HASTINGS
-                if (printDebugInfo) System.err.print(logAlpha + " " + newLogLikelihood + " " + oldLogLikelihood);
-
-                if (logAlpha >= 0 || Randomizer.nextDouble() < Math.exp(logAlpha)) {
-                    // accept
-                    oldLogLikelihood = newLogLikelihood;
-                    state.acceptCalculationNodes();
-
-                    if (sampleNr >= 0) {
-                        operator.accept();
-                    }
-                    if (printDebugInfo) System.err.print(" accept");
-                } else {
-                    // reject
-                    if (sampleNr >= 0) {
-                        operator.reject(newLogLikelihood == Double.NEGATIVE_INFINITY ? -1 : 0);
-                    }
-                    state.restore();
-                    state.restoreCalculationNodes();
-                    if (printDebugInfo) System.err.print(" reject");
-                }
-                state.setEverythingDirty(false);
-            } else {
-                // operation failed
-                if (sampleNr >= 0) {
-                    operator.reject(-2);
-                }
-                state.restore();
-				if (!operator.requiresStateInitialisation()) {
-                    state.setEverythingDirty(false);
-                    state.restoreCalculationNodes();
-				}
-				if (printDebugInfo) System.err.print(" direct reject");
-            }
-            log(sampleNr);
+        for (long sampleNr = -burnIn; sampleNr <= chainLength; sampleNr++) {
+            final Operator operator = propagateState(sampleNr);
 
             if (debugFlag && sampleNr % 3 == 0 || sampleNr % 10000 == 0) {
                 // check that the posterior is correctly calculated at every third
@@ -475,7 +391,7 @@ public class MCMC extends Runnable {
                     reportLogLikelihoods(posterior, "");
                     Log.err.println("At sample " + sampleNr + "\nLikelihood incorrectly calculated: " + originalLogP + " != " + logLikelihood
                     		+ "(" + (originalLogP - logLikelihood) + ")"
-                            + " Operator: " + operator.getClass().getName());
+                            + " Operator: " + operator.getName());
                 }
                 if (sampleNr > NR_OF_DEBUG_SAMPLES * 3) {
                     // switch off debug mode once a sufficient large sample is checked
@@ -515,12 +431,108 @@ public class MCMC extends Runnable {
                 state.storeToFile(sampleNr);
                 operatorSchedule.storeToFile();
             }
+            
+            if (posterior.getCurrentLogP() == Double.POSITIVE_INFINITY) {
+            	throw new RuntimeException("Encountered a positive infinite posterior. This is a sign there may be numeric instability in the model.");
+            }
         }
         if (corrections > 0) {
         	Log.err.println("\n\nNB: " + corrections + " posterior calculation corrections were required. This analysis may not be valid!\n\n");
         }
     }
 
+    /**
+     * Perform a single MCMC propose+accept/reject step.
+     *
+     * @param sampleNr the index of the current MCMC step
+     * @return the selected {@link beast.core.Operator}
+     */
+    protected Operator propagateState(final long sampleNr) {
+        state.store(sampleNr);
+//            if (m_nStoreEvery > 0 && sample % m_nStoreEvery == 0 && sample > 0) {
+//                state.storeToFile(sample);
+//            	operatorSchedule.storeToFile();
+//            }
+
+        final Operator operator = operatorSchedule.selectOperator();
+
+        if (printDebugInfo) System.err.print("\n" + sampleNr + " " + operator.getName()+ ":");
+
+        final Distribution evaluatorDistribution = operator.getEvaluatorDistribution();
+        Evaluator evaluator = null;
+
+        if (evaluatorDistribution != null) {
+            evaluator = new Evaluator() {
+                @Override
+                public double evaluate() {
+                    double logP = 0.0;
+
+                    state.storeCalculationNodes();
+                    state.checkCalculationNodesDirtiness();
+
+                    try {
+                        logP = evaluatorDistribution.calculateLogP();
+                    } catch (Exception e) {
+                        e.printStackTrace();
+                        System.exit(1);
+                    }
+
+                    state.restore();
+                    state.store(sampleNr);
+
+                    return logP;
+                }
+            };
+        }
+        final double logHastingsRatio = operator.proposal(evaluator);
+
+        if (logHastingsRatio != Double.NEGATIVE_INFINITY) {
+
+            if (operator.requiresStateInitialisation()) {
+                state.storeCalculationNodes();
+                state.checkCalculationNodesDirtiness();
+            }
+
+            newLogLikelihood = posterior.calculateLogP();
+
+            logAlpha = newLogLikelihood - oldLogLikelihood + logHastingsRatio; //CHECK HASTINGS
+            if (printDebugInfo) System.err.print(logAlpha + " " + newLogLikelihood + " " + oldLogLikelihood);
+
+            if (logAlpha >= 0 || Randomizer.nextDouble() < Math.exp(logAlpha)) {
+                // accept
+                oldLogLikelihood = newLogLikelihood;
+                state.acceptCalculationNodes();
+
+                if (sampleNr >= 0) {
+                    operator.accept();
+                }
+                if (printDebugInfo) System.err.print(" accept");
+            } else {
+                // reject
+                if (sampleNr >= 0) {
+                    operator.reject(newLogLikelihood == Double.NEGATIVE_INFINITY ? -1 : 0);
+                }
+                state.restore();
+                state.restoreCalculationNodes();
+                if (printDebugInfo) System.err.print(" reject");
+            }
+            state.setEverythingDirty(false);
+        } else {
+            // operation failed
+            if (sampleNr >= 0) {
+                operator.reject(-2);
+            }
+            state.restore();
+            if (!operator.requiresStateInitialisation()) {
+                state.setEverythingDirty(false);
+                state.restoreCalculationNodes();
+            }
+            if (printDebugInfo) System.err.print(" direct reject");
+        }
+        log(sampleNr);
+        return operator;
+    }
+
     private boolean isTooDifferent(double logLikelihood, double originalLogP) {
     	//return Math.abs((logLikelihood - originalLogP)/originalLogP) > 1e-6;
     	return Math.abs(logLikelihood - originalLogP) > 1e-6;
@@ -534,7 +546,7 @@ public class MCMC extends Runnable {
     protected void reportLogLikelihoods(final Distribution distr, final String tabString) {
         final double full =  distr.logP, last = distr.storedLogP;
         final String changed = full == last ? "" : "  **";
-        Log.err.println(tabString + "P(" + distr.getID() + ") = " + full + " (was " + last + ")" + changed);
+        Log.info.println(tabString + "P(" + distr.getID() + ") = " + full + " (was " + last + ")" + changed);
         if (distr instanceof CompoundDistribution) {
             for (final Distribution distr2 : ((CompoundDistribution) distr).pDistributions.get()) {
                 reportLogLikelihoods(distr2, tabString + "\t");
@@ -542,7 +554,7 @@ public class MCMC extends Runnable {
         }
     }
 
-    protected void callUserFunction(final int sample) {
+    protected void callUserFunction(final long sample) {
     }
 
 
