@@ -24,14 +24,6 @@
 */
 package beast.evolution.alignment;
 
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collections;
-import java.util.Comparator;
-import java.util.HashSet;
-import java.util.List;
-import java.util.Set;
-
 import beast.core.Description;
 import beast.core.Input;
 import beast.core.Input.Validate;
@@ -39,7 +31,9 @@ import beast.core.parameter.Map;
 import beast.core.util.Log;
 import beast.evolution.datatype.DataType;
 import beast.evolution.datatype.StandardData;
-import beast.util.AddOnManager;
+import beast.util.PackageManager;
+
+import java.util.*;
 
 @Description("Class representing alignment data")
 public class Alignment extends Map<String> {
@@ -70,7 +64,7 @@ public class Alignment extends Map<String> {
 
     static public void findDataTypes() {
         // build up list of data types
-        List<String> m_sDataTypes = AddOnManager.find(beast.evolution.datatype.DataType.class, IMPLEMENTATION_DIR);
+        List<String> m_sDataTypes = PackageManager.find(beast.evolution.datatype.DataType.class, IMPLEMENTATION_DIR);
         for (String dataTypeName : m_sDataTypes) {
             try {
                 DataType dataType = (DataType) Class.forName(dataTypeName).newInstance();
@@ -107,6 +101,10 @@ public class Alignment extends Map<String> {
     final public Input<Integer> excludetoInput = new Input<>("excludeto", "last site to condition on (but excluding this site), default 0", 0);
     final public Input<Integer> excludeeveryInput = new Input<>("excludeevery", "interval between sites to condition on (default 1)", 1);
 
+	public Input<Integer> m_includefrom = new Input<>("includefrom","first site to condition on, default 0", 0);
+	public Input<Integer> m_includeto = new Input<>("includeto","last site to condition on, default 0", 0);
+	public Input<Integer> m_includeevery = new Input<>("includeevery","interval between sites to condition on (default 1)", 1);
+
     /**
      * list of sequences in the alignment *
      */
@@ -153,7 +151,7 @@ public class Alignment extends Map<String> {
      * characters are uncertain.
      */
     public List<double[][]> tipLikelihoods = new ArrayList<>(); // #taxa x #sites x #states
-    private boolean usingTipLikelihoods = false;
+    protected boolean usingTipLikelihoods = false;
     
     /**
      * pattern state encodings *
@@ -169,6 +167,7 @@ public class Alignment extends Map<String> {
      * From AscertainedAlignment
      */
     Set<Integer> excludedPatterns;
+	List<Integer> m_nIncluded;
 
     /**
      * A flag to indicate if the alignment is ascertained
@@ -226,24 +225,7 @@ public class Alignment extends Map<String> {
         if (userDataTypeInput.get() != null) {
             m_dataType = userDataTypeInput.get();
         } else {
-            if (types.indexOf(dataTypeInput.get()) < 0) {
-                throw new IllegalArgumentException("data type + '" + dataTypeInput.get() + "' cannot be found. " +
-                        "Choose one of " + Arrays.toString(types.toArray(new String[0])));
-            }
-            // seems to spend forever in there??
-            List<String> dataTypes = AddOnManager.find(beast.evolution.datatype.DataType.class, IMPLEMENTATION_DIR);
-            for (String dataTypeName : dataTypes) {
-                DataType dataType;
-				try {
-					dataType = (DataType) Class.forName(dataTypeName).newInstance();
-	                if (dataTypeInput.get().equals(dataType.getTypeDescription())) {
-	                    m_dataType = dataType;
-	                    break;
-	                }
-				} catch (InstantiationException | IllegalAccessException | ClassNotFoundException e) {
-					throw new IllegalArgumentException(e.getMessage());
-				}
-            }
+            initDataType();
         }
 
         // initialize the sequence list
@@ -269,6 +251,31 @@ public class Alignment extends Map<String> {
         Log.info.println(toString(false));
     }
 
+    /**
+     * Initializes data types using
+     * {@link PackageManager#find(Class, String[]) PackageManager.find}
+     */
+    protected void initDataType() {
+        if (types.indexOf(dataTypeInput.get()) < 0) {
+            throw new IllegalArgumentException("data type + '" + dataTypeInput.get() + "' cannot be found. " +
+                    "Choose one of " + Arrays.toString(types.toArray(new String[0])));
+        }
+        // seems to spend forever in there??
+        List<String> dataTypes = PackageManager.find(DataType.class, IMPLEMENTATION_DIR);
+        for (String dataTypeName : dataTypes) {
+            DataType dataType;
+            try {
+                dataType = (DataType) Class.forName(dataTypeName).newInstance();
+                if (dataTypeInput.get().equals(dataType.getTypeDescription())) {
+                    m_dataType = dataType;
+                    break;
+                }
+            } catch (InstantiationException | IllegalAccessException | ClassNotFoundException e) {
+                throw new IllegalArgumentException(e.getMessage());
+            }
+        }
+    }
+
     /**
      * Initializes the alignment given the provided list of sequences and no other information.
      * It site weights and/or data type have been previously set up with initAndValidate then they
@@ -307,7 +314,7 @@ public class Alignment extends Map<String> {
             }
         } catch (Exception e) {
             e.printStackTrace();
-            System.exit(1);
+            throw new RuntimeException(e);
         }
         sanityCheckCalcPatternsSetUpAscertainment(log);
     }
@@ -315,7 +322,7 @@ public class Alignment extends Map<String> {
     /**
      * Checks that sequences are all the same length, calculates patterns and sets up ascertainment.
      */
-    private void sanityCheckCalcPatternsSetUpAscertainment(boolean log) {
+    protected void sanityCheckCalcPatternsSetUpAscertainment(boolean log) {
         // Sanity check: make sure sequences are of same length
         int length = counts.get(0).size();
         if (!(m_dataType instanceof StandardData)) {
@@ -365,6 +372,18 @@ public class Alignment extends Map<String> {
                 patternWeight[patternIndex_] = 0;
                 excludedPatterns.add(patternIndex_);
             }
+            
+    		from = m_includefrom.get();
+    		to = m_includeto.get();
+    		every = m_includeevery.get();
+    		m_nIncluded = new ArrayList<>();
+    		for (int i = from; i < to; i += every) {
+    			int patternIndex_ = patternIndex[i];
+    			// reduce weight, so it does not confuse the tree likelihood
+    			patternWeight[patternIndex_] = 0;
+    			m_nIncluded.add(patternIndex_);
+    		}
+
         } else {
         	// sanity check
             int from = excludefromInput.get();
@@ -386,7 +405,7 @@ public class Alignment extends Map<String> {
             states[i] = sitePattern[taxonIndex];
         }
         try {
-            return data.getDataType().state2string(states);
+            return data.getDataType().encodingToString(states);
         } catch (Exception e) {
             e.printStackTrace();
             System.exit(1);
@@ -701,7 +720,7 @@ public class Alignment extends Map<String> {
     }
 
     boolean isAmbiguousState(int state) {
-        return (state >= 0 && state < maxStateCount);
+        return (state < 0 || state >= maxStateCount);
     }
 
     //Methods from AscertainedAlignment
@@ -713,9 +732,13 @@ public class Alignment extends Map<String> {
         return excludedPatterns.size();
     }
 
-    public double getAscertainmentCorrection(double[] patternLogProbs) {
+	public double getAscertainmentCorrection(double[] patternLogProbs) {
         double excludeProb = 0, includeProb = 0, returnProb = 1.0;
 
+        for (int i = 0; i < m_nIncluded.size(); i++) {
+        	includeProb += Math.exp(patternLogProbs[m_nIncluded.get(i)]);
+        }
+
         for (int i : excludedPatterns) {
             excludeProb += Math.exp(patternLogProbs[i]);
         }
@@ -725,7 +748,7 @@ public class Alignment extends Map<String> {
         } else if (excludeProb == 0.0) {
             returnProb = includeProb;
         } else {
-            returnProb = includeProb - excludeProb;
+            returnProb = 1.0 + includeProb - excludeProb;
         }
         return Math.log(returnProb);
     } // getAscertainmentCorrection
@@ -757,7 +780,7 @@ public class Alignment extends Map<String> {
 		}
 		String seq = null;
 		try {
-			seq = m_dataType.state2string(states);
+			seq = m_dataType.encodingToString(states);
 		} catch (Exception e) {
 			e.printStackTrace();
 		}
