@@ -52,7 +52,9 @@ public class TreeLikelihood extends GenericTreeLikelihood {
 
     final public Input<Boolean> m_useAmbiguities = new Input<>("useAmbiguities", "flag to indicate that sites containing ambiguous states should be handled instead of ignored (the default)", false);
     final public Input<Boolean> m_useTipLikelihoods = new Input<>("useTipLikelihoods", "flag to indicate that partial likelihoods are provided at the tips", false);
-    
+    final public Input<String> implementationInput = new Input<>("implementation", "name of class that implements this treelikelihood potentially more efficiently. "
+    		+ "This class will be tried first, with the TreeLikelihood as fallback implementation. "
+    		+ "When multi-threading, multiple objects can be created.", "beast.evolution.likelihood.BeagleTreeLikelihood");
     
     public static enum Scaling {none, always, _default};
     final public Input<Scaling> scaling = new Input<>("scaling", "type of scaling to use, one of " + Arrays.toString(Scaling.values()) + ". If not specified, the -beagle_scaling flag is used.", Scaling._default, Scaling.values());
@@ -62,7 +64,7 @@ public class TreeLikelihood extends GenericTreeLikelihood {
      * calculation engine *
      */
     protected LikelihoodCore likelihoodCore;
-    BeagleTreeLikelihood beagle;
+    protected BeagleTreeLikelihood beagle;
 
     /**
      * BEASTObject associated with inputs. Since none of the inputs are StateNodes, it
@@ -103,14 +105,14 @@ public class TreeLikelihood extends GenericTreeLikelihood {
     /**
      * memory allocation for probability tables obtained from the SiteModel *
      */
-    double[] probabilities;
+    protected double[] probabilities;
 
-    int matrixSize;
+    protected int matrixSize;
 
     /**
      * flag to indicate ascertainment correction should be applied *
      */
-    boolean useAscertainedSitePatterns = false;
+    protected boolean useAscertainedSitePatterns = false;
 
     /**
      * dealing with proportion of site being invariant *
@@ -209,7 +211,7 @@ public class TreeLikelihood extends GenericTreeLikelihood {
             Arrays.fill(isInvariant, true);
             for (final int state : pattern) {
                 final boolean[] isStateSet = dataInput.get().getStateSet(state);
-                if (m_useAmbiguities.get() || !dataInput.get().getDataType().isAmbiguousState(state)) {
+                if (m_useAmbiguities.get() || !dataInput.get().getDataType().isAmbiguousCode(state)) {
                     for (int k = 0; k < stateCount; k++) {
                         isInvariant[k] &= isStateSet[k];
                     }
@@ -473,6 +475,14 @@ public class TreeLikelihood extends GenericTreeLikelihood {
         return update;
     } // traverseWithBRM
 
+    /* return copy of pattern log likelihoods for each of the patterns in the alignment */
+	public double [] getPatternLogLikelihoods() {
+		if (beagle != null) {
+			return beagle.getPatternLogLikelihoods();
+		}
+		return patternLogLikelihoods.clone();
+	} // getPatternLogLikelihoods
+
     /** CalculationNode methods **/
 
     /**
