@@ -1,19 +1,15 @@
 package beast.evolution.tree;
 
+import beast.core.*;
+import beast.core.util.Log;
+import beast.evolution.alignment.TaxonSet;
+import beast.util.TreeParser;
+
 import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 
-import beast.core.Description;
-import beast.core.Input;
-import beast.core.Operator;
-import beast.core.StateNode;
-import beast.core.StateNodeInitialiser;
-import beast.core.util.Log;
-import beast.evolution.alignment.TaxonSet;
-import beast.util.TreeParser;
-
 
 @Description("Tree (the T in BEAST) representing gene beast.tree, species"
         + " beast.tree, language history, or other time-beast.tree"
@@ -199,6 +195,15 @@ public class Tree extends StateNode implements TreeInterface {
     public Tree() {
     }
 
+    /**
+     * This constructor wraps a new root node in a <code>Tree</code> object, which does not copy the tree.
+     * To copy a <code>Tree</code>, either {@link Tree#copy() Tree#copy} or {@link Node#copy() Node#copy}
+     * can be used instead, which performs true deep copy.
+     * For example, <code>new Tree(oldTree.getRoot().copy())</code> which is also equivalent to
+     * <code>oldTree.copy()</code>.
+     *
+     * @param rootNode root <code>Node</code>
+     */
     public Tree(final Node rootNode) {
         setRoot(rootNode);
         initArrays();
@@ -340,7 +345,8 @@ public class Tree extends StateNode implements TreeInterface {
                  m_sTaxaNames = new String[getNodeCount()];
                  collectTaxaNames(getRoot());
                  List<String> taxaNames = new ArrayList<>();
-                 for (String name : m_sTaxaNames) {
+                 for (int i=0; i<m_sTaxaNames.length && i<getLeafNodeCount(); i++) {
+                     String name = m_sTaxaNames[i];
                      if (name != null) {
                          taxaNames.add(name);
                      }
@@ -630,8 +636,7 @@ public class Tree extends StateNode implements TreeInterface {
 
     @Override
     public int scale(final double scale) {
-        root.scale(scale);
-        return getInternalNodeCount()- getDirectAncestorNodeCount();
+        return root.scale(scale);
     }
 
 
@@ -714,7 +719,7 @@ public class Tree extends StateNode implements TreeInterface {
     }
 
     @Override
-	public void log(int sample, PrintStream out) {
+	public void log(long sample, PrintStream out) {
         Tree tree = (Tree) getCurrent();
         out.print("tree STATE_" + sample + " = ");
         // Don't sort, this can confuse CalculationNodes relying on the tree
@@ -929,6 +934,17 @@ public class Tree extends StateNode implements TreeInterface {
             return height;
     }
 
+    /**
+     * @return a string describing what type of times/ages are being used in MRCA priors for this tree.
+     *
+     */
+    public String getDateType() {
+        if (hasDateTrait()) {
+            return timeTraitSet.getDateType();
+        } else
+            return "age";
+    }
+
     /**
      * This method allows the retrieval of the taxon label of a node without using the node number.
      *
@@ -979,6 +995,11 @@ public class Tree extends StateNode implements TreeInterface {
             leafNodeCount++;
     }
 
+    /**
+     * @return the number of direct ancestors in this tree. Direct ancestor nodes conceptually have a single child. However
+     * this is determined by whether the node's parent has the same height as the node, since that signifies a zero branch length
+     * and the parent node with the same height as the child node is regarded as "fake".
+     */
     public int getDirectAncestorNodeCount() {
         int directAncestorNodeCount = 0;
         for (int i = 0; i < leafNodeCount; i++) {
@@ -994,4 +1015,22 @@ public class Tree extends StateNode implements TreeInterface {
     public TaxonSet getTaxonset() {
         return m_taxonset.get();
     }
+    
+    
+    /** 
+     * Determine whether a child was replaced in a binary tree 
+     **/
+    public boolean childrenChanged(int nodeNr) {
+        Node node = m_nodes[nodeNr];
+        Node old = m_storedNodes[nodeNr];
+        if (node.getLeft().getNr() == old.getLeft().getNr() &&
+                node.getRight().getNr() == old.getRight().getNr()) {
+                return false;
+        }
+        if (node.getLeft().getNr() == old.getRight().getNr() &&
+                node.getRight().getNr() == old.getLeft().getNr()) {
+                return false;
+        }
+        return true;
+    }
 } // class Tree
\ No newline at end of file
