@@ -25,8 +25,6 @@
 package beast.util;
 
 
-import static beast.util.XMLParserUtils.processPlates;
-import static beast.util.XMLParserUtils.replaceVariable;
 
 import java.io.File;
 import java.io.IOException;
@@ -71,6 +69,8 @@ import beast.evolution.alignment.Alignment;
 import beast.evolution.alignment.Sequence;
 import beast.evolution.tree.Tree;
 
+import static beast.util.XMLParserUtils.*;
+
 
 /**
  * XMLParser process Beast 2 XML and constructs an MCMC object
@@ -251,13 +251,23 @@ public class XMLParser {
      */
     RequiredInputProvider requiredInputProvider = null;
     PartitionContext partitionContext = null;
+	java.util.Map<String,String> parserDefinitions;
 
     public XMLParser() {
-        beastObjectsWaitingToInit = new ArrayList<>();
-        nodesWaitingToInit = new ArrayList<>();
+        this(new HashMap<>());
     }
 
+    public XMLParser(java.util.Map<String,String> parserDefinitions) {
+        beastObjectsWaitingToInit = new ArrayList<>();
+        nodesWaitingToInit = new ArrayList<>();
+        this.parserDefinitions = parserDefinitions;
+	}
+
     public Runnable parseFile(final File file) throws SAXException, IOException, ParserConfigurationException, XMLParserException {
+    	return parseFile(file, false);
+    }
+    
+    public Runnable parseFile(final File file, boolean sampleFromPrior) throws SAXException, IOException, ParserConfigurationException, XMLParserException {
         // parse the XML file into a DOM document
         final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
         //factory.setValidating(true);
@@ -265,19 +275,34 @@ public class XMLParser {
         doc.normalize();
         processPlates(doc,PLATE_ELEMENT);
 
-        // Substitute occurrences of "$(filebase)" with name of file 
-        int pointIdx = file.getName().lastIndexOf('.');
-        String baseName = pointIdx<0 ? file.getName() : file.getName().substring(0, pointIdx);
-        if (doc.getElementsByTagName(BEAST_ELEMENT).item(0) == null) {
+        Node beastElement = doc.getElementsByTagName(BEAST_ELEMENT).item(0);
+
+        // Sanity check
+        if (beastElement == null) {
         	Log.err.println("Incorrect XML: Could not find 'beast' element in file " + file.getName());
         	throw new RuntimeException();
         }
-        replaceVariable(doc.getElementsByTagName(BEAST_ELEMENT).item(0), "filebase", baseName);
 
-        // Substitute occurrences of "$(seed)" with RNG seed
-        replaceVariable(doc.getElementsByTagName(BEAST_ELEMENT).item(0), "seed",
-                String.valueOf(Randomizer.getSeed()));
-        
+        // Add special variables "filebase" (name of file excluding extension)
+        // and "seed" (RNG seed) to list of user-defined variables.
+        int pointIdx = file.getName().lastIndexOf('.');
+        String baseName = pointIdx<0 ? file.getName() : file.getName().substring(0, pointIdx);
+        parserDefinitions.put("filebase", baseName);
+        parserDefinitions.put("seed", String.valueOf(Randomizer.getSeed()));
+
+
+		if (sampleFromPrior) {
+			Element runElement = (Element) doc.getElementsByTagName(RUN_ELEMENT).item(0);
+	        runElement.setAttribute("sampleFromPrior", "true");
+		}
+		
+
+        // Extract default values of variables if present
+        extractVariableDefaults(beastElement, parserDefinitions);
+
+        // Replace occurrences of variables with their corresponding values
+        replaceVariables(beastElement, parserDefinitions);
+
         IDMap = new HashMap<>();
         likelihoodMap = new HashMap<>();
         IDNodeMap = new HashMap<>();
@@ -495,7 +520,7 @@ public class XMLParser {
         		if (i > 0) {
         			String pkgname = s.substring(0, i);
         			String pkgversion = s.substring(i+1);
-        			if (!AddOnManager.isInstalled(pkgname, pkgversion)) {
+        			if (!PackageManager.isInstalled(pkgname, pkgversion)) {
         				unavailablePacakges += s +", ";
         			}
         		}
@@ -619,7 +644,7 @@ public class XMLParser {
             this.nameSpaces[i++] = nameSpace;
         }
         // make sure that the default namespace is in there
-        this.nameSpaces[i] = "";
+        this.nameSpaces[i] = "";        
     }
 
     void parseRunElement(final Node topNode) throws XMLParserException {
@@ -683,9 +708,13 @@ public class XMLParser {
             }
             if (IDMap.containsKey(dRef)) {
                 final BEASTInterface beastObject = IDMap.get(dRef);
-                if (checkType(classname, beastObject, node)) {
+                // TODO: testing for "Alignment" is a hack for a common problem
+                // occurring in many templates. As long as packages are not debugged
+                // this hack should stay in place, but should eventually (v2.5?) be removed.
+                if (classname.equals("Alignment") || checkType(classname, beastObject, node)) {
                     return beastObject;
                 }
+                checkType(classname, beastObject, node);
                 throw new XMLParserException(node, "id=" + dRef + ". Expected object of type " + classname + " instead of " + beastObject.getClass().getName(), 106);
             } else if (IDNodeMap.containsKey(dRef)) {
                 final BEASTInterface beastObject = createObject(IDNodeMap.get(dRef), classname);
@@ -715,14 +744,7 @@ public class XMLParser {
         
 		String clazzName = null;
 		// determine clazzName from specName, taking name spaces in account
-		for (String nameSpace : nameSpaces) {
-			if (clazzName == null) {
-				if (XMLParserUtils.beastObjectNames.contains(nameSpace + specClass)) {
-					clazzName = nameSpace + specClass;
-					break;
-				}
-			}
-		}
+		clazzName = resolveClass(specClass);
 		if (clazzName == null) {
 			// try to create the old-fashioned way by creating the class
             boolean isDone = false;
@@ -759,7 +781,7 @@ public class XMLParser {
 				throw new XMLParserException(node, "Expected object to be instance of BEASTObject", 108);
 			}
 		} catch (ClassNotFoundException e1) {
-			// should never happen since clazzName is in the list of classes collected by the AddOnManager
+			// should never happen since clazzName is in the list of classes collected by the PackageManager
 			e1.printStackTrace();
 			throw new RuntimeException(e1);
 		}
@@ -771,6 +793,7 @@ public class XMLParser {
         // initialise
         if (needsInitialisation) {
             try {
+            	beastObject.determindClassOfInputs();
                 beastObject.validateInputs();
                 beastObjectsWaitingToInit.add(beastObject);
                 nodesWaitingToInit.add(node);
@@ -915,6 +938,18 @@ public class XMLParser {
 		    				((List)args[i]).addAll(values);
 		    			} else {
 		    				args[i] = getValue(param, types[i], inputInfo);
+		    				// deal with the case where the Input type has a String constructor
+		    				// and the args[i] is a String -- we need to invoke the String constructor 
+		    				if (args[i].getClass().equals(String.class) && types[i] != String.class) {
+		    				    for (Constructor<?> argctor : types[i].getDeclaredConstructors()) {
+		    				    	Class<?>[] argtypes  = argctor.getParameterTypes();
+		    				    	if (argtypes.length == 1 && argtypes[0] == String.class) {
+		    				    		Object o = argctor.newInstance(args[i]);
+		    				    		args[i] = o;
+		    				    		break;
+		    				    	}
+		    				    }
+		    				}
 		    			}
 		    		}
 
@@ -1055,10 +1090,21 @@ public class XMLParser {
                     name = element;
                 }
                 // resolve base class
-                String classname = BEAST_INTERFACE_CLASS;
+                String specClass = BEAST_INTERFACE_CLASS;
                 if (element2ClassMap.containsKey(element)) {
-                    classname = element2ClassMap.get(element);
+                	specClass = element2ClassMap.get(element);
                 }
+                final String spec = getAttribute(child, "spec");
+                if (spec != null) {
+                	specClass = spec;
+                }                
+                String classname = null;
+        		// determine clazzName from specName, taking name spaces in account
+                classname = resolveClass(specClass);
+        		if (classname == null) {
+        			classname = specClass;
+        		}
+        		
                 // test for special cases: <xyz>value</xyz>  and <input name="xyz">value</input>
                 // where value is a string
                 boolean done = false;
@@ -1109,6 +1155,26 @@ public class XMLParser {
 
         return inputInfo;
     } // setInputs
+    
+    private String resolveClass(String specClass) {
+		for (String nameSpace : nameSpaces) {
+			if (XMLParserUtils.beastObjectNames.contains(nameSpace + specClass)) {
+				String clazzName = nameSpace + specClass;
+				return clazzName;
+			}
+		}
+		for (String nameSpace : nameSpaces) {
+            try {
+				if (Class.forName(nameSpace + specClass) != null) {
+					String clazzName = nameSpace + specClass;
+					return clazzName;
+				}
+			} catch (ClassNotFoundException e) {
+				// ignore
+			}
+		}
+		return null;
+	}
 
     void setInput(final Node node, final BEASTInterface beastObject, final String name, final BEASTInterface beastObject2) throws XMLParserException {
         try {
@@ -1119,9 +1185,12 @@ public class XMLParser {
             if (input.get() == input.defaultValue) {
                 beastObject.setInputValue(name, beastObject2);
             } else {
-                throw new XMLParserException("Multiple entries for non-list input " + input.getName());
+                throw new XMLParserException(node, "\nMultiple entries for input \"" + input.getName() + "\" but only single entry expected "
+                		+ "in element \"" + node.getNodeName() + "\"", 130);
             }
             return;
+        } catch (XMLParserException e) {
+        	throw e;
         } catch (Exception e) {
         	if (name.equals("xml:base")) {
         		// ignore xml:base attributes introduces by XML entities
